<?xml version="1.0" encoding="UTF-8" ?>
<?xml-stylesheet type="text/xsl" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat"
		xmlns:md="http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat"
		xmlns:tsx="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat"
		xmlns:dc="http://purl.org/dc/elements/1.1/"
		xsi:schemaLocation="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat EATimeSeriesDataExchangeFormat.xsd http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat EAMetadata.xsd">

<!-- =================================== -->

	<dc:Title>Long Term Probability Ranking (v2.0)</dc:Title>
	<dc:Date>2008-03-12</dc:Date>
	<dc:Description>Creates sets-of-values for each long-term-statistical probability band (e.g. Normal, Minimum, Extremely High) and writes them to a new XML output, with the productRef and pointReference being used to identify each band. The bands are interpolated from the ranked data, with the date given for the more extreme value. Note that XML-annotations are written to the output file indicating the values on either side of the threshold.</dc:Description>
	<dc:Relation>This must be run in Microsoft environment as it uses the msxsl extension for node sets.</dc:Relation>
	<dc:Creator>Chris Beales</dc:Creator>
	<dc:Format>text/xsl</dc:Format>
	<dc:Identifier>http://websites.chrisbeales.net/TSX/tsx-Trans_LTA Probability Bands.xsl</dc:Identifier>
	<dc:Language>en</dc:Language>
	<dc:Publisher>Chris Beales, Environment Agency</dc:Publisher>
	<dc:Rights>Copyright 2006-08 Environment Agency</dc:Rights>
	<dc:Type>XSL Stylesheet</dc:Type>
	<dc:Subject>XML, EATimeSeriesDataExchangeFormat, TSX, Envrionment Agency Management System (AMS): Categorising Hydrological Data by Probability Ranking</dc:Subject>

<!-- =================================== -->

	<xsl:variable name="XHigh">0.95</xsl:variable>
	<xsl:variable name="NoteHigh">0.87</xsl:variable>
	<xsl:variable name="High">0.72</xsl:variable>
	<xsl:variable name="Low">0.28</xsl:variable>
	<xsl:variable name="NoteLow">0.13</xsl:variable>
	<xsl:variable name="XLow">0.05</xsl:variable>

<!-- =================================== -->

	<xsl:output encoding="UTF-8" version="1.0" method="xml" indent="yes" />
	
	<xsl:template match="/">
		<xsl:apply-templates />
		
	</xsl:template>

	<xsl:template match="@*|node()"> 	
		<xsl:copy>
			<xsl:apply-templates select="@*|node()" />
		</xsl:copy>
	</xsl:template>

	<xsl:template match="tsx:SetofValues">	
		<xsl:variable name="NoOfYears" select="number(substring(@endDate, 1, 4)) - number(substring(@startDate, 1, 4))" />

		<xsl:variable name="DateList">
			<xsl:apply-templates select="tsx:Value" mode="GetDates" />
		</xsl:variable>		

		<xsl:variable name="UniqueDates" select="msxsl:node-set($DateList)/child::node()[not(. = preceding-sibling::*)]" />

		<xsl:variable name="VarValue" select="tsx:Value" />

	<!-- Rank Maximum -->
		<xsl:element name="SetofValues">		
			<xsl:for-each select="@*">
				<xsl:if test="(name() != 'pointReference') and (name() != 'productRef')">
					<xsl:attribute name="{name()}">
						<xsl:value-of select="." />
					</xsl:attribute>
				</xsl:if>
			</xsl:for-each>

			<xsl:attribute name="productRef">
				<xsl:value-of select="'Maximum'" />
			</xsl:attribute>

			<xsl:attribute name="pointReference">
				<xsl:value-of select="concat('Rank ', $NoOfYears, ' of ', $NoOfYears)" />
			</xsl:attribute>

			<xsl:for-each select="$UniqueDates">
				<xsl:sort select="." />

				<xsl:variable name="TestDate" select="." />

				<xsl:if test="$TestDate != '02-29'">
					<xsl:for-each select="msxsl:node-set($VarValue)[substring(@date, 6) = $TestDate]">
						<xsl:sort select="." data-type="number" />

						<xsl:if test="position() = last()">
							<xsl:copy-of select="." />
						</xsl:if>
					</xsl:for-each>
				</xsl:if>
			</xsl:for-each>
		</xsl:element>

	<!-- Rank Extremely High -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Ex High'" />
			<xsl:with-param name="Threshold" select="$XHigh" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'True'" />
		</xsl:call-template>

	<!-- Rank Notably High -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Note High'" />
			<xsl:with-param name="Threshold" select="$NoteHigh" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'True'" />
		</xsl:call-template>

	<!-- Above Normal -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Above Norm'" />
			<xsl:with-param name="Threshold" select="$High" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'True'" />
		</xsl:call-template>

	<!-- Below Normal -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Below Norm'" />
			<xsl:with-param name="Threshold" select="$Low" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'False'" />
		</xsl:call-template>

	<!-- Notably Low -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Note Low'" />
			<xsl:with-param name="Threshold" select="$NoteLow" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'False'" />
		</xsl:call-template>

	<!-- Rank Extremely Low -->
		<xsl:call-template name="RankSOV">
			<xsl:with-param name="ProductName" select="'Ex Low'" />
			<xsl:with-param name="Threshold" select="$XLow" />
			<xsl:with-param name="sourceSOV" select="msxsl:node-set(.)" />
			<xsl:with-param name="NoOfYears" select="$NoOfYears" />
			<xsl:with-param name="UniqueDates" select="$UniqueDates" />
			<xsl:with-param name="VarValue" select="tsx:Value" />
			<xsl:with-param name="bolHigh" select="'False'" />
		</xsl:call-template>

	<!-- Rank Minimum -->
		<xsl:element name="SetofValues">		
		<xsl:for-each select="@*">
				<xsl:if test="(name() != 'pointReference') and (name() != 'productRef')">
					<xsl:attribute name="{name()}">
						<xsl:value-of select="." />
					</xsl:attribute>
				</xsl:if>
			</xsl:for-each>

			<xsl:attribute name="productRef">
				<xsl:value-of select="'Minimum'" />
			</xsl:attribute>

			<xsl:attribute name="pointReference">
				<xsl:value-of select="concat('Rank 1 of ', $NoOfYears)" />
			</xsl:attribute>

			<xsl:for-each select="$UniqueDates">
				<xsl:sort select="." />

				<xsl:variable name="TestDate" select="." />

				<xsl:if test="$TestDate != '02-29'">
					<xsl:for-each select="msxsl:node-set($VarValue)[substring(@date, 6) = $TestDate]">
						<xsl:sort select="." data-type="number" />

						<xsl:if test="position() = 1">
							<xsl:copy-of select="." />
						</xsl:if>
					</xsl:for-each>
				</xsl:if>
			</xsl:for-each>
		</xsl:element>

	</xsl:template>

<!-- Get month-day dates -->
	<xsl:template match="tsx:Value" mode="GetDates">
		<xsl:element name="MoDa">
			<xsl:value-of select="msxsl:node-set(substring(@date, 6))" />
		</xsl:element>
	</xsl:template>
	
<!-- Carry out ranking for set-of-values -->
	<xsl:template name="RankSOV">
		<xsl:param name="ProductName" />
		<xsl:param name="Threshold" />
		<xsl:param name="sourceSOV" />
		<xsl:param name="NoOfYears" />
		<xsl:param name="UniqueDates" />
		<xsl:param name="VarValue" />
		<xsl:param name="bolHigh" />

		<xsl:element name="SetofValues">		
			<xsl:for-each select="$sourceSOV/@*">
				<xsl:if test="(name() != 'pointReference') and (name() != 'productRef')">
					<xsl:attribute name="{name()}">
						<xsl:value-of select="." />
					</xsl:attribute>
				</xsl:if>
			</xsl:for-each>
			
			<xsl:variable name="TempAbove" select="ceiling(($Threshold * ($NoOfYears + '.2')) + '.4')" />
			<xsl:variable name="RankAbove">
				<xsl:choose>
					<xsl:when test="$TempAbove = 0">
						<xsl:value-of select="'1'" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$TempAbove" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="ThresholdAbove" select="($RankAbove - '0.4') div ($NoOfYears + '.2')" />

			<xsl:variable name="TempBelow" select="floor(($Threshold * ($NoOfYears + '.2')) + '.4')" />
			<xsl:variable name="RankBelow">
				<xsl:choose>
					<xsl:when test="$TempBelow = 0">
						<xsl:value-of select="'1'" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$TempBelow" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="ThresholdBelow" select="($RankBelow - '0.4') div ($NoOfYears + '.2')" />

			<xsl:variable name="Ratio" select="($Threshold - $ThresholdBelow) div ($ThresholdAbove - $ThresholdBelow)" />

			<xsl:variable name="RankNumber" select="$RankBelow + $Ratio" />
			
			<xsl:attribute name="productRef">
				<xsl:value-of select="$ProductName" />
			</xsl:attribute>

			<xsl:attribute name="pointReference">
				<xsl:value-of select="concat('Rank ', (round($RankNumber * 100) div 100), ' of ', $NoOfYears, ' : Threshold = ', $Threshold)" />
			</xsl:attribute>
			
			<xsl:for-each select="$UniqueDates">
				<xsl:sort select="." />

				<xsl:variable name="TestDate" select="." />

				<xsl:if test="$TestDate != '02-29'">

					<xsl:variable name="ValueAbove">
						<xsl:for-each select="msxsl:node-set($VarValue)[substring(@date, 6) = $TestDate]">
							<xsl:sort select="." data-type="number" />

							<xsl:if test="position() = $RankAbove">
								<xsl:copy-of select="." />
							</xsl:if>
						</xsl:for-each>
					</xsl:variable>

					<xsl:variable name="ValueBelow">
						<xsl:for-each select="msxsl:node-set($VarValue)[substring(@date, 6) = $TestDate]">
							<xsl:sort select="." data-type="number" />

							<xsl:if test="position() = $RankBelow">
								<xsl:copy-of select="." />
							</xsl:if>
						</xsl:for-each>
					</xsl:variable>

					<xsl:comment>
						<xsl:value-of select="concat('Rank ', $RankAbove, ', Threshold ', (round($ThresholdAbove * 10000) div 10000), ', ', msxsl:node-set($ValueAbove)/tsx:Value/@date, ', ', msxsl:node-set($ValueAbove))" />
						<xsl:if test="msxsl:node-set($ValueAbove)/tsx:Value/@flag1">
							<xsl:value-of select="concat(', quality ', msxsl:node-set($ValueAbove)/tsx:Value/@flag1)" />
						</xsl:if>
					</xsl:comment>
					<xsl:comment>
						<xsl:value-of select="concat('Rank ', $RankBelow, ', Threshold ', (round($ThresholdBelow * 10000) div 10000), ', ', msxsl:node-set($ValueBelow)/tsx:Value/@date, ', ', msxsl:node-set($ValueBelow))" />
						<xsl:if test="msxsl:node-set($ValueBelow)/tsx:Value/@flag1">
							<xsl:value-of select="concat(', quality ', msxsl:node-set($ValueBelow)/tsx:Value/@flag1)" />
						</xsl:if>
					</xsl:comment>

					<xsl:element name="Value">
						<xsl:attribute name="date">
							<xsl:choose>
								<xsl:when test="$bolHigh = 'True'">
									<xsl:value-of select="msxsl:node-set($ValueAbove)/tsx:Value/@date" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="msxsl:node-set($ValueBelow)/tsx:Value/@date" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>

						<xsl:value-of select="round((($Ratio * ($ValueAbove - $ValueBelow)) + $ValueBelow) * 10000) div 10000" />
					</xsl:element>

				</xsl:if>
			</xsl:for-each>	
		</xsl:element>
	</xsl:template>

</xsl:stylesheet>

