<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" ?>
<xsl:stylesheet version="1.0" 
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
		xmlns="http://www.w3.org/1999/xhtml"
		xmlns:tsx="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat" 
		xmlns:md="http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:dc="http://purl.org/dc/elements/1.1/"
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
		xsi:schemaLocation="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat EATimeSeriesDataExchangeFormat.xsd http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat EAMetadata.xsd">

<!-- =================================== -->

	<dc:Title>CSV Table Generator for LTA Probability Bands (v1.1)</dc:Title>
	<dc:Date>2008-03-12</dc:Date>
	<dc:Description>Takes the output the Long Term Probability Ranking stylesheet and generates a table in CSV format (in an XHTML wrapper). This can then be used to map against data values in a table or as the backdrop to a graph.</dc:Description>
	<dc:Relation>This stylesheet is designed to mark-up the results of the {tsx-Trans_LTA Probability Bands.xsl} stylesheet. This must be run in Microsoft environment as it uses the msxsl extension for node sets. Note that the {tsx_Monthly Data comparison v1'0.xls} is designed to graph the results of this stylesheet.</dc:Relation>
	<dc:Creator>Chris Beales</dc:Creator>
	<dc:Format>text/xsl</dc:Format>
	<dc:Identifier>http://websites.chrisbeales.net/TSX/tsx-Trans_Table of Probability Bands (CSV).xsl</dc:Identifier>
	<dc:Language>en</dc:Language>
	<dc:Publisher>Chris Beales, Environment Agency</dc:Publisher>
	<dc:Rights>Copyright 2006-08 Environment Agency</dc:Rights>
	<dc:Type>XSL Stylesheet</dc:Type>
	<dc:Subject>XML, EATimeSeriesDataExchangeFormat, TSX, XHTML, HTML, CSV, Table</dc:Subject>

<!-- =================================== -->

	<xsl:output 
		method="xml" 
		version="1.0" 
		encoding="iso-8859-1" 
		indent="no" 
		omit-xml-declaration="no"
		doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
		doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
		media-type="text/html" />

	<xsl:template match="/">
		<xsl:element name="html">
			<xsl:attribute name="xml:lang">en</xsl:attribute>
			<xsl:attribute name="lang">en</xsl:attribute>

			<xsl:element name="head">
				<xsl:element name="title">Time-series data file</xsl:element>
			</xsl:element>  <!-- head -->

			<xsl:element name="body">
				<xsl:element name="pre">		<!-- preserve -->
				<!-- Header -->
<xsl:text
>Tabulated LTA probability bands from {tsx-Trans_LTA Probability Bands.xsl} mark-up

</xsl:text>

				<!-- Main -->

					<xsl:apply-templates select="tsx:EATimeSeriesDataExchangeFormat"/>

				</xsl:element>  <!-- pre -->
			</xsl:element>  <!-- body -->
		</xsl:element>  <!-- html -->		
	</xsl:template>

<!-- Main document -->
	<xsl:template match="tsx:EATimeSeriesDataExchangeFormat">
		<xsl:value-of select="concat('Publisher,', md:Publisher, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Source,', md:Source, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Description,', md:Description, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Creator,', md:Creator, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Date,', substring(md:Date, 9), '/', substring(md:Date, 6, 2), '/', substring(md:Date, 1, 4), '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Time,', md:Time, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Identifier,', md:Identifier, '&#xd;&#xA;')" />

		<xsl:choose>
			<xsl:when test="md:Publisher='Environment Agency'">
				<xsl:text>
Disclaimer: "The information provided is based on that currently available to the Agency. 
The Agency and its officers accept no liability whatsoever for any loss or damage arising 
from the interpretation or use of the information." &#xd;&#xA;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'&#xd;&#xA;&#xd;&#xA;&#xd;&#xA;&#xd;&#xA;'" />
			</xsl:otherwise>
		</xsl:choose>

<xsl:text>
========================================================

</xsl:text>

		<xsl:apply-templates select="tsx:Station" />

	</xsl:template>

<!-- Station element -->
	<xsl:template match="tsx:Station">
		<xsl:value-of select="concat('Station Reference,', @stationReference, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Station Name,', @stationName, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Region,', @region, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('National Grid Reference,', @ngr, '&#xd;&#xA;&#xd;&#xA;')"/>

		<xsl:value-of select="concat('Parameter,', tsx:SetofValues[1]/@parameter, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Qualifier,', tsx:SetofValues[1]/@qualifier, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Data Type,', tsx:SetofValues[1]/@dataType, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Period,', tsx:SetofValues[1]/@period, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Interval,', tsx:SetofValues[1]/@interval, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Characteristic,', tsx:SetofValues[1]/@characteristic, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Units,', tsx:SetofValues[1]/@units, '&#xd;&#xA;')"/>

		<xsl:variable name="vStartTime">
			<xsl:if test="tsx:SetofValues[1]/@startTime">
				<xsl:value-of select="concat(' ', tsx:SetofValues[1]/@startTime)" />
			</xsl:if>
		</xsl:variable>
		<xsl:value-of select="concat('Start Date,', substring(tsx:SetofValues[1]/@startDate, 9), '/', substring(tsx:SetofValues[1]/@startDate, 6, 2), '/', substring(tsx:SetofValues[1]/@startDate, 1, 4), $vStartTime, '&#xd;&#xA;')"/>

		<xsl:variable name="vEndTime">
			<xsl:if test="tsx:SetofValues[1]/@endTime">
				<xsl:value-of select="concat(' ', tsx:SetofValues[1]/@endTime)" />
			</xsl:if>
		</xsl:variable>	
		<xsl:value-of select="concat('End Date,', substring(tsx:SetofValues[1]/@endDate, 9), '/', substring(tsx:SetofValues[1]/@endDate, 6, 2), '/', substring(tsx:SetofValues[1]/@endDate, 1, 4), $vEndTime, '&#xd;&#xA;')"/>

		<xsl:value-of select="concat('Day Origin,', tsx:SetofValues[1]/@dayOrigin, '&#xd;&#xA;&#xd;&#xA;')"/>

	<!-- Table -->
		<xsl:value-of select="'Probability Band,'"/>
		<xsl:for-each select="tsx:SetofValues">
			<xsl:value-of select="@productRef"/>
			<xsl:if test="position() != last()">
				<xsl:value-of select="','" />
			</xsl:if>
		</xsl:for-each>
		<xsl:value-of select="'&#xd;&#xA;'"/>

		<xsl:value-of select="'Statistics,'"/>
		<xsl:for-each select="tsx:SetofValues">
			<xsl:value-of select="@pointReference"/>
			<xsl:if test="position() != last()">
				<xsl:value-of select="','" />
			</xsl:if>
		</xsl:for-each>
		<xsl:value-of select="'&#xd;&#xA;'"/>


		<xsl:variable name="UniqueDateList">
			<xsl:apply-templates select="tsx:SetofValues[1]/tsx:Value" mode="GetDates" />
		</xsl:variable>		

		<xsl:variable name="VarStation" select="." />

		<xsl:for-each select="msxsl:node-set($UniqueDateList)/*">
			<xsl:sort select="." />
			
			<xsl:variable name="MoDa" select="." />

			<xsl:value-of select="concat('2000-', $MoDa, ',')"/>

			<xsl:for-each select="msxsl:node-set($VarStation)/tsx:SetofValues">
				<xsl:value-of select="tsx:Value[substring(@date, 6) = $MoDa]"/>
				<xsl:if test="position() != last()">
					<xsl:value-of select="','" />
				</xsl:if>				
			</xsl:for-each>
			
			<xsl:value-of select="'&#xd;&#xA;'"/>
		</xsl:for-each>

	</xsl:template>

<!-- Get month-day dates -->
	<xsl:template match="tsx:Value" mode="GetDates">
		<xsl:element name="MoDa">
			<xsl:value-of select="msxsl:node-set(substring(@date, 6))" />
		</xsl:element>
	</xsl:template>
	
</xsl:stylesheet>
